@@ -230,6 +230,7 @@ class SubGraph:
 
                     'degrees': 1,
 
+                    'edges': True,
                     'filterinput': True,
                     'yieldfiltered': False,
 
@@ -1282,6 +1283,11 @@ class PivotOut(PivotOper):
             if form is None:
                 continue
 
+            if prop.isrunt:
+                async for pivo in runt.snap.nodesByPropValu(form.name, '=', valu):
+                    yield pivo, path.fork(pivo)
+                continue
+
             pivo = await runt.snap.getNodeByNdef((form.name, valu))
             if pivo is None:  # pragma: no cover
                 continue
@@ -1662,10 +1668,14 @@ class PropPivotOut(PivotOper):
 
             prop = node.form.props.get(name)
             if prop is None:
+                # all filters must sleep
+                await asyncio.sleep(0)
                 continue
 
             valu = node.get(name)
             if valu is None:
+                # all filters must sleep
+                await asyncio.sleep(0)
                 continue
 
             if prop.type.isarray:
@@ -1731,6 +1741,8 @@ class PropPivot(PivotOper):
 
             srcprop, valu = await self.kids[0].getPropAndValu(path)
             if valu is None:
+                # all filters must sleep
+                await asyncio.sleep(0)
                 continue
 
             # TODO cache/bypass normalization in loop!
@@ -2618,7 +2630,9 @@ class TagName(RunValue):
     async def compute(self, path):
         if self.isconst:
             return self.value()
-        vals = [(await kid.compute(path)) for kid in self.kids]
+        vals = []
+        for kid in self.kids:
+            vals.append(await tostr(await kid.compute(path)))
         return '.'.join(vals)
 
 class TagMatch(TagName):
