@@ -36,9 +36,9 @@ def _fini_atexit(): # pragma: no cover
 
         if not item._fini_atexit and not OMIT_FINI_WARNS:
             if __debug__:
-                print(f'At exit: Missing fini for {item}')
+                logger.debug(f'At exit: Missing fini for {item}')
                 for depth, call in enumerate(item.call_stack[:-2]):
-                    print(f'{depth+1:3}: {call.strip()}')
+                    logger.debug(f'{depth+1:3}: {call.strip()}')
             continue
 
         try:
@@ -129,11 +129,6 @@ class Base:
         self.anitted = True  # For assertion purposes
         self.finievt = None
         self.entered = False
-        self.exitinfo = None
-
-        self.exitok = None
-        self.entered = False
-        self.exitinfo = None
 
         # hold a weak ref to other bases we should fini if they
         # are still around when we go down...
@@ -210,20 +205,8 @@ class Base:
         except RuntimeError:
             pass
 
-        self.exitok = cls is None
-        self.exitinfo = (exc, cls, tb)
         await self.fini()
 
-    def _isExitExc(self):
-        # if entered but not exited *or* exitinfo has exc
-        if not self.entered:
-            return False
-
-        if self.exitinfo is None:
-            return True
-
-        return self.exitinfo[0] is not None
-
     def incref(self):
         '''
         Increment the reference count for this base.  This API may be optionally used to control fini().
