@@ -1,9 +1,12 @@
 import os
 import ssl
+import time
+import shutil
 import socket
 import asyncio
 import logging
 import argparse
+import datetime
 import functools
 import contextlib
 
@@ -28,9 +31,10 @@ import synapse.lib.certdir as s_certdir
 import synapse.lib.httpapi as s_httpapi
 import synapse.lib.version as s_version
 import synapse.lib.hiveauth as s_hiveauth
-
 import synapse.lib.lmdbslab as s_lmdbslab
 
+import synapse.tools.backup as s_t_backup
+
 logger = logging.getLogger(__name__)
 
 SLAB_MAP_SIZE = 128 * s_const.mebibyte
@@ -90,7 +94,7 @@ class CellApi(s_base.Base):
 
             Form a path and check the permission from a remote proxy::
 
-                perm = ('node:add', 'inet:ipv4')
+                perm = ('node', 'add', 'inet:ipv4')
                 allowed = await prox.allowed(perm)
                 if allowed:
                     dostuff()
@@ -137,6 +141,14 @@ class CellApi(s_base.Base):
     def getCellIden(self):
         return self.cell.getCellIden()
 
+    @adminapi()
+    def getNexsIndx(self):
+        return self.cell.getNexsIndx()
+
+    @adminapi()
+    async def promote(self):
+        return await self.cell.promote()
+
     def getCellUser(self):
         return self.user.pack()
 
@@ -436,6 +448,40 @@ class CellApi(s_base.Base):
         async for item in self.cell.getNexusChanges(offs):
             yield item
 
+    @adminapi()
+    async def runBackup(self, name=None, wait=True):
+        '''
+        Run a new backup.
+
+        Args:
+            name (str): The optional name of the backup.
+            wait (bool): On True, wait for backup to complete before returning.
+
+        Returns:
+            str: The name of the newly created backup.
+        '''
+        return await self.cell.runBackup(name=name, wait=wait)
+
+    @adminapi()
+    async def getBackups(self):
+        '''
+        Retrieve a list of backups.
+
+        Returns:
+            list[str]: A list of backup names.
+        '''
+        return await self.cell.getBackups()
+
+    @adminapi()
+    async def delBackup(self, name):
+        '''
+        Delete a backup by name.
+
+        Args:
+            name (str): The name of the backup to delete.
+        '''
+        return await self.cell.delBackup(name)
+
     @adminapi()
     async def getDiagInfo(self):
         return {
@@ -445,6 +491,14 @@ class CellApi(s_base.Base):
 class Cell(s_nexus.Pusher, s_telepath.Aware):
     '''
     A Cell() implements a synapse micro-service.
+
+    A Cell has 5 phases of startup:
+        1. Universal cell data structures
+        2. Service specific storage/data (pre-nexs)
+        3. Nexus subsystem initialization
+        4. Service specific startup (with nexus)
+        5. Networking and mirror services
+
     '''
     cellapi = CellApi
 
@@ -467,25 +521,43 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
             'description': 'Record all changes to the cell.  Required for mirroring (on both sides).',
             'type': 'boolean'
         },
+        'nexslog:async': {
+            'default': False,
+            'description': '(Experimental) Map the nexus log LMDB instance with map_async=True.',
+            'type': 'boolean',
+        },
+        'dmon:listen': {
+            'description': 'A config-driven way to specify the telepath bind URL.',
+            'type': ['string', 'null'],
+        },
+        'https:port': {
+            'description': 'A config-driven way to specify the HTTPS port.',
+            'type': ['integer', 'null'],
+        },
+        'backup:dir': {
+            'description': 'A directory outside the service directory where backups will be saved.',
+            'type': 'string',
+        },
     }
 
-    async def __anit__(self, dirn, conf=None, readonly=False, *args, **kwargs):
+    async def __anit__(self, dirn, conf=None, readonly=False):
 
+        # phase 1
         if conf is None:
             conf = {}
 
         s_telepath.Aware.__init__(self)
-
         self.dirn = s_common.gendir(dirn)
 
         self.auth = None
         self.sessions = {}
+        self.isactive = False
         self.inaugural = False
 
         self.conf = self._initCellConf(conf)
 
         # each cell has a guid
-        path = s_common.genpath(dirn, 'cell.guid')
+        path = s_common.genpath(self.dirn, 'cell.guid')
 
         # generate a guid file if needed
         if not os.path.isfile(path):
@@ -505,12 +577,33 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
 
         self.donexslog = self.conf.get('nexslog:en')
 
+        backdirn = self.conf.get('backup:dir')
+        if backdirn is not None:
+            backdirn = s_common.genpath(backdirn)
+            if backdirn.startswith(self.dirn):
+                mesg = 'backup:dir must not be within the service directory'
+                raise s_exc.BadConfValu(mesg=mesg)
+
+            backdirn = s_common.gendir(backdirn)
+
+        self.backdirn = backdirn
+
+        if self.conf.get('mirror') and not self.conf.get('nexslog:en'):
+            mesg = 'Mirror mode requires nexslog:en=True'
+            raise s_exc.BadConfValu(mesg=mesg)
+
+        # construct our nexsroot instance ( but do not start it )
         await s_nexus.Pusher.__anit__(self, self.iden)
 
-        await self._initCellSlab(readonly=readonly)
+        root = await self._ctorNexsRoot()
 
-        self.setNexsRoot(await self._initNexsRoot())
-        self.nexsroot.onStateChange(self.onLeaderChange)
+        # mutually assured destruction with our nexs root
+        self.onfini(root.fini)
+        root.onfini(self.fini)
+
+        self.setNexsRoot(root)
+
+        await self._initCellSlab(readonly=readonly)
 
         self.hive = await self._initCellHive()
 
@@ -536,61 +629,180 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
             if not user.tryPasswd(auth_passwd):
                 await user.setPasswd(auth_passwd, nexs=False)
 
-        await self._initCellDmon()
-
         self.boss = await s_boss.Boss.anit()
         self.onfini(self.boss)
 
-        await self._initCellHttp()
-
-        self._health_funcs = []
-        self.addHealthFunc(self._cellHealth)
-
-        async def fini():
-            [await s.fini() for s in self.sessions.values()]
-
-        self.onfini(fini)
-
         self.dynitems = {
             'auth': self.auth,
             'cell': self
         }
 
-    async def postAnit(self):
-        mirror = self.conf.get('mirror')
-        await self.nexsroot.setLeader(mirror, self.iden)
+        # initialize web app and callback data structures
+        self._health_funcs = []
+        self.addHealthFunc(self._cellHealth)
 
-    async def _initNexsRoot(self):
+        # initialize network daemons (but do not listen yet)
+        # to allow registration of callbacks and shared objects
+        # within phase 2/4.
+        await self._initCellHttp()
+        await self._initCellDmon()
+
+        # phase 2 - service storage
+        await self.initServiceStorage()
+        # phase 3 - nexus subsystem
+        await self.initNexusSubsystem()
+        # phase 4 - service logic
+        await self.initServiceRuntime()
+        # phase 5 - service networking
+        await self.initServiceNetwork()
+
+    async def initServiceStorage(self):
+        pass
+
+    async def initNexusSubsystem(self):
+        mirror = self.conf.get('mirror')
+        await self.nexsroot.startup(mirror, celliden=self.iden)
+        await self.setCellActive(mirror is None)
+
+    async def initServiceNetwork(self):
+
+        # start a unix local socket daemon listener
+        sockpath = os.path.join(self.dirn, 'sock')
+        sockurl = f'unix://{sockpath}'
+
+        try:
+            await self.dmon.listen(sockurl)
+        except asyncio.CancelledError:  # pragma: no cover
+            raise
+        except OSError as e:
+            logger.error(f'Failed to listen on unix socket at: [{sockpath}][{e}]')
+            logger.error('LOCAL UNIX SOCKET WILL BE UNAVAILABLE')
+        except Exception:  # pragma: no cover
+            logging.exception(f'Unknown dmon listen error.')
+            raise
+
+        turl = self.conf.get('dmon:listen')
+        if turl is not None:
+            await self.dmon.listen(turl)
+            logger.info(f'dmon listening: {turl}')
+
+        port = self.conf.get('https:port')
+        if port is not None:
+            await self.addHttpsPort(port)
+            logger.info(f'https listening: {port}')
+
+    async def initServiceRuntime(self):
+        pass
+
+    async def _ctorNexsRoot(self):
         '''
         Initialize a NexsRoot to use for the cell.
         '''
-        nexsroot = await s_nexus.NexsRoot.anit(self.dirn, donexslog=self.donexslog)
-        self.onfini(nexsroot.fini)
-        nexsroot.onfini(self)
-        await nexsroot.setLeader(None, '')
-        return nexsroot
+        map_async = self.conf.get('nexslog:async')
+        return await s_nexus.NexsRoot.anit(self.dirn, donexslog=self.donexslog, map_async=map_async)
 
-    async def onLeaderChange(self, leader):
+    async def getNexsIndx(self):
+        return await self.nexsroot.index()
+
+    async def promote(self):
         '''
-        Args:
-            leader(bool):  If True, self is now the leader, else is now a follower
+        Transform this cell from a passive follower to
+        an active cell that writes changes locally.
         '''
-        self.isleader = leader
-        if leader:
-            await self.startAsLeader()
+        if self.conf.get('mirror') is None:
+            mesg = 'promote() called on non-mirror'
+            raise s_exc.BadConfValu(mesg=mesg)
+
+        await self.nexsroot.promote()
+        await self.setCellActive(True)
+
+    async def setCellActive(self, active):
+        self.isactive = active
+        if self.isactive:
+            await self.initServiceActive()
         else:
-            await self.stopAsLeader()
+            await self.initServicePassive()
 
-    async def startAsLeader(self):
+    async def initServiceActive(self): # pragma: no cover
         pass
 
-    async def stopAsLeader(self):
+    async def initServicePassive(self): # pragma: no cover
         pass
 
     async def getNexusChanges(self, offs):
         async for item in self.nexsroot.iter(offs):
             yield item
 
+    def _reqBackDirn(self, name):
+        self._reqBackConf()
+
+        path = s_common.genpath(self.backdirn, name)
+        if not path.startswith(self.backdirn):
+            mesg = 'Directory traversal detected'
+            raise s_exc.BadArg(mesg=mesg)
+
+        return path
+
+    async def runBackup(self, name=None, wait=True):
+
+        if name is None:
+            name = time.strftime('%Y%m%d%H%M%S', datetime.datetime.now().timetuple())
+
+        path = self._reqBackDirn(name)
+        if os.path.isdir(path):
+            mesg = 'Backup with name already exists'
+            raise s_exc.BadArg(mesg=mesg)
+
+        task = self.schedCoro(self._execBackTask(path))
+
+        if wait:
+            await task
+
+        return name
+
+    async def _execBackTask(self, dirn):
+        await self.boss.promote('backup', self.auth.rootuser)
+        todo = s_common.todo(s_t_backup.backup, self.dirn, dirn)
+        await s_coro.spawn(todo)
+
+    def _reqBackConf(self):
+        if self.backdirn is None:
+            mesg = 'Backup APIs require the backup:dir config option is set'
+            raise s_exc.NeedConfValu(mesg=mesg)
+
+    async def delBackup(self, name):
+
+        self._reqBackConf()
+        path = self._reqBackDirn(name)
+
+        cellguid = os.path.join(path, 'cell.guid')
+        if not os.path.isfile(cellguid):
+            mesg = 'Specified backup path has no cell.guid file.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        await s_coro.executor(shutil.rmtree, path, ignore_errors=True)
+
+    async def getBackups(self):
+        self._reqBackConf()
+        backups = []
+
+        def walkpath(path):
+
+            for name in os.listdir(path):
+
+                full = os.path.join(path, name)
+                cellguid = os.path.join(full, 'cell.guid')
+
+                if os.path.isfile(cellguid):
+                    backups.append(os.path.relpath(full, self.backdirn))
+                    continue
+
+                if os.path.isdir(full):
+                    walkpath(full)
+
+        walkpath(self.backdirn)
+        return backups
+
     async def isUserAllowed(self, iden, perm, gateiden=None):
         user = self.auth.user(iden)
         if user is None:
@@ -832,6 +1044,7 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
         self.sessstor = s_lmdbslab.GuidStor(self.slab, 'http:sess')
 
         async def fini():
+            [await s.fini() for s in self.sessions.values()]
             for http in self.httpds:
                 http.stop()
 
@@ -876,24 +1089,11 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
         ))
 
     async def _initCellDmon(self):
-        # start a unix local socket daemon listener
-        sockpath = os.path.join(self.dirn, 'sock')
-        sockurl = f'unix://{sockpath}'
+        cdir = s_common.gendir(self.dirn, 'certs')
 
-        self.dmon = await s_daemon.Daemon.anit()
+        self.dmon = await s_daemon.Daemon.anit(certdir=(cdir, s_certdir.defdir))
         self.dmon.share('*', self)
 
-        try:
-            await self.dmon.listen(sockurl)
-        except asyncio.CancelledError:  # pragma: no cover
-            raise
-        except OSError as e:
-            logger.error(f'Failed to listen on unix socket at: [{sockpath}][{e}]')
-            logger.error('LOCAL UNIX SOCKET WILL BE UNAVAILABLE')
-        except Exception:  # pragma: no cover
-            logging.exception(f'Unknown dmon listen error.')
-            raise
-
         self.onfini(self.dmon.fini)
 
     async def _initCellHive(self):
@@ -1105,16 +1305,34 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
 
         try:
 
-            await cell.addHttpsPort(opts.https)
-            await cell.dmon.listen(opts.telepath)
+            if 'dmon:listen' not in cell.conf:
+                await cell.dmon.listen(opts.telepath)
+                if outp is not None:
+                    outp.printf(f'...{cell.getCellType()} API (telepath): %s' % (opts.telepath,))
+            else:
+
+                if outp is not None:
+                    lisn = cell.conf.get('dmon:listen')
+                    if lisn is None:
+                        lisn = cell.getLocalUrl()
+
+                    outp.printf(f'...{cell.getCellType()} API (telepath): %s' % (lisn,))
+
+            if 'https:port' not in cell.conf:
+                await cell.addHttpsPort(opts.https)
+                if outp is not None:
+                    outp.printf(f'...{cell.getCellType()} API (https): %s' % (opts.https,))
+            else:
+                if outp is not None:
+                    port = cell.conf.get('https:port')
+                    if port is None:
+                        outp.printf(f'...{cell.getCellType()} API (https): disabled')
+                    else:
+                        outp.printf(f'...{cell.getCellType()} API (https): %s' % (port,))
 
             if opts.name is not None:
                 cell.dmon.share(opts.name, cell)
-
-            if outp is not None:
-                outp.printf(f'...{cell.getCellType()} API (telepath): %s' % (opts.telepath,))
-                outp.printf(f'...{cell.getCellType()} API (https): %s' % (opts.https,))
-                if opts.name is not None:
+                if outp is not None:
                     outp.printf(f'...{cell.getCellType()} API (telepath name): %s' % (opts.name,))
 
         except Exception:
