@@ -63,7 +63,6 @@ import struct
 import asyncio
 import logging
 import ipaddress
-import itertools
 import contextlib
 import collections
 
@@ -79,6 +78,7 @@ import synapse.lib.cell as s_cell
 import synapse.lib.cache as s_cache
 import synapse.lib.nexus as s_nexus
 import synapse.lib.queue as s_queue
+import synapse.lib.urlhelp as s_urlhelp
 
 import synapse.lib.config as s_config
 import synapse.lib.lmdbslab as s_lmdbslab
@@ -94,7 +94,8 @@ reqValidLdef = s_config.getJsValidator({
         'iden': {'type': 'string', 'pattern': s_config.re_iden},
         'creator': {'type': 'string', 'pattern': s_config.re_iden},
         'lockmemory': {'type': 'boolean'},
-        'logedits': {'type': 'boolean'}, 'default': True
+        'logedits': {'type': 'boolean'}, 'default': True,
+        'name': {'type': 'string'},
     },
     'additionalProperties': True,
     'required': ['iden', 'creator', 'lockmemory'],
@@ -141,14 +142,14 @@ class LayerApi(s_cell.CellApi):
 
         await self.layr.storNodeEditsNoLift(nodeedits, meta)
 
-    async def syncNodeEdits(self, offs):
+    async def syncNodeEdits(self, offs, wait=True):
         '''
         Yield (offs, nodeedits) tuples from the nodeedit log starting from the given offset.
 
         Once caught up with storage, yield them in realtime.
         '''
         await self._reqUserAllowed(self.liftperm)
-        async for item in self.layr.syncNodeEdits(offs):
+        async for item in self.layr.syncNodeEdits(offs, wait=wait):
             yield item
 
     async def splices(self, offs=None, size=None):
@@ -200,6 +201,7 @@ STOR_TYPE_I128 = 20
 STOR_TYPE_MINTIME = 21
 
 STOR_TYPE_FLOAT64 = 22
+STOR_TYPE_HUGENUM = 23
 
 # STOR_TYPE_TOMB      = ??
 # STOR_TYPE_FIXED     = ??
@@ -649,6 +651,66 @@ class StorTypeInt(StorType):
         for item in liftby.buidsByRange(pkeymin, pkeymax):
             yield item
 
+class StorTypeHugeNum(StorType):
+
+    def __init__(self, layr, stortype):
+        StorType.__init__(self, layr, STOR_TYPE_HUGENUM)
+        self.lifters.update({
+            '=': self._liftHugeEq,
+            '<': self._liftHugeLt,
+            '>': self._liftHugeGt,
+            '<=': self._liftHugeLe,
+            '>=': self._liftHugeGe,
+            'range=': self._liftHugeRange,
+        })
+
+        self.one = s_common.hugenum(1)
+        self.offset = s_common.hugenum(0x7fffffffffffffffffffffffffffffff)
+
+        self.zerobyts = b'\x00' * 16
+        self.fullbyts = b'\xff' * 16
+
+    def getHugeIndx(self, norm):
+        scaled = s_common.hugenum(norm).scaleb(15)
+        byts = int(scaled + self.offset).to_bytes(16, byteorder='big')
+        return byts
+
+    def indx(self, norm):
+        return (self.getHugeIndx(norm),)
+
+    async def _liftHugeEq(self, liftby, valu):
+        byts = self.getHugeIndx(valu)
+        for item in liftby.buidsByDups(byts):
+            yield item
+
+    async def _liftHugeGt(self, liftby, valu):
+        valu = s_common.hugenum(valu)
+        async for item in self._liftHugeGe(liftby, valu + self.one):
+            yield item
+
+    async def _liftHugeLt(self, liftby, valu):
+        valu = s_common.hugenum(valu)
+        async for item in self._liftHugeLe(liftby, valu - self.one):
+            yield item
+
+    async def _liftHugeGe(self, liftby, valu):
+        pkeymin = self.getHugeIndx(valu)
+        pkeymax = self.fullbyts
+        for item in liftby.buidsByRange(pkeymin, pkeymax):
+            yield item
+
+    async def _liftHugeLe(self, liftby, valu):
+        pkeymin = self.zerobyts
+        pkeymax = self.getHugeIndx(valu)
+        for item in liftby.buidsByRange(pkeymin, pkeymax):
+            yield item
+
+    async def _liftHugeRange(self, liftby, valu):
+        pkeymin = self.getHugeIndx(valu[0])
+        pkeymax = self.getHugeIndx(valu[1])
+        for item in liftby.buidsByRange(pkeymin, pkeymax):
+            yield item
+
 class StorTypeFloat(StorType):
     FloatPacker = struct.Struct('>d')
     fpack = FloatPacker.pack
@@ -700,7 +762,6 @@ class StorTypeFloat(StorType):
             yield item[1]
 
     async def _liftFloatGt(self, liftby, valu):
-        genr = self._liftFloatGeCommon(liftby, valu)
         valupack = self.fpack(valu)
         async for item in self._liftFloatGeCommon(liftby, valu):
             if item[0] == valupack:
@@ -727,7 +788,6 @@ class StorTypeFloat(StorType):
             yield item[1]
 
     async def _liftFloatLt(self, liftby, valu):
-        genr = self._liftFloatLeCommon(liftby, valu)
         valupack = self.fpack(valu)
         async for item in self._liftFloatLeCommon(liftby, valu):
             if item[0] == valupack:
@@ -977,6 +1037,7 @@ class Layer(s_nexus.Pusher):
             StorTypeTime(self), # STOR_TYPE_MINTIME
 
             StorTypeFloat(self, STOR_TYPE_FLOAT64, 8),
+            StorTypeHugeNum(self, STOR_TYPE_HUGENUM),
         ]
 
         self.editors = [
@@ -1017,7 +1078,9 @@ class Layer(s_nexus.Pusher):
 
     @s_nexus.Pusher.onPushAuto('layer:truncate')
     async def truncate(self):
-
+        '''
+        Nuke all the contents in the layer, leaving an empty layer
+        '''
         self.buidcache.clear()
 
         await self.layrslab.trash()
@@ -1026,6 +1089,46 @@ class Layer(s_nexus.Pusher):
 
         await self._initLayerStorage()
 
+    async def clone(self, newdirn):
+        '''
+        Copy the contents of this layer to a new layer
+        '''
+        for root, dnames, fnames in os.walk(self.dirn, topdown=True):
+
+            relpath = os.path.relpath(root, start=self.dirn)
+
+            for name in list(dnames):
+
+                relname = os.path.join(relpath, name)
+
+                srcpath = s_common.genpath(root, name)
+                dstpath = s_common.genpath(newdirn, relname)
+
+                if srcpath in s_lmdbslab._AllSlabs:
+                    slab = s_lmdbslab._AllSlabs.get(srcpath)
+                    await slab.copyslab(dstpath)
+
+                    dnames.remove(name)
+                    continue
+
+                s_common.gendir(dstpath)
+
+            for name in fnames:
+
+                srcpath = s_common.genpath(root, name)
+                # skip unix sockets etc...
+                if not os.path.isfile(srcpath):
+                    continue
+
+                dstpath = s_common.genpath(newdirn, relpath, name)
+                shutil.copy(srcpath, dstpath)
+
+    async def waitForHot(self):
+        '''
+        Wait for the layer's slab to be prefaulted and locked into memory if lockmemory is true, otherwise return.
+        '''
+        await self.layrslab.lockdoneevent.wait()
+
     async def _initLayerStorage(self):
 
         slabopts = {
@@ -1073,9 +1176,7 @@ class Layer(s_nexus.Pusher):
         self.nodedata = self.dataslab.initdb('nodedata')
         self.dataname = self.dataslab.initdb('dataname', dupsort=True)
 
-        self.nodeeditlog = None
-        if self.logedits:
-            self.nodeeditlog = self.nodeeditctor(self.nodeeditslab, 'nodeedits')
+        self.nodeeditlog = self.nodeeditctor(self.nodeeditslab, 'nodeedits')
 
     def getSpawnInfo(self):
         info = self.pack()
@@ -1154,6 +1255,11 @@ class Layer(s_nexus.Pusher):
             return None
         return s_msgpack.un(byts)
 
+    async def getNodeForm(self, buid):
+        byts = self.layrslab.get(buid + b'\x09', db=self.bybuid)
+        if byts is not None:
+            return byts.decode()
+
     async def getStorNode(self, buid):
         '''
         Return a potentially incomplete pode.
@@ -1198,6 +1304,31 @@ class Layer(s_nexus.Pusher):
 
         return (buid, info)
 
+    async def getTagCount(self, tagname, formname=None):
+        '''
+        Return the number of tag rows in the layer for the given tag/form.
+        '''
+        try:
+            abrv = self.tagabrv.bytsToAbrv(tagname.encode())
+            if formname is not None:
+                abrv += self.getPropAbrv(formname, None)
+
+        except s_exc.NoSuchAbrv:
+            return 0
+
+        return await self.layrslab.countByPref(abrv, db=self.bytag)
+
+    async def getPropCount(self, formname, propname=None):
+        '''
+        Return the number of property rows in the layer for the given form/prop.
+        '''
+        try:
+            abrv = self.getPropAbrv(formname, propname)
+        except s_exc.NoSuchAbrv:
+            return 0
+
+        return await self.layrslab.countByPref(abrv, db=self.byprop)
+
     async def liftByTag(self, tag, form=None):
 
         try:
@@ -1324,8 +1455,8 @@ class Layer(s_nexus.Pusher):
 
         return retn
 
-    @s_nexus.Pusher.onPush('edits')
-    async def _storNodeEdits(self, nodeedits, meta):
+    @s_nexus.Pusher.onPush('edits', passitem=True)
+    async def _storNodeEdits(self, nodeedits, meta, nexsitem):
         '''
         Execute a series of node edit operations, returning the updated nodes.
 
@@ -1336,27 +1467,29 @@ class Layer(s_nexus.Pusher):
             List[Tuple[buid, form, edits]]  Same list, but with only the edits actually applied (plus the old value)
         '''
         results = []
+        nexsindx = nexsitem[0]
 
-        for edits in [await self._storNodeEdit(e) for e in nodeedits]:
+        for edits in [await self._storNodeEdit(e, meta) for e in nodeedits]:
             results.extend(edits)
 
         if self.logedits:
+
             changes = [r for r in results if r[2]]
             if changes:
-
-                offs = self.nodeeditlog.add((changes, meta))
+                offs = self.nodeeditlog.add((changes, meta), indx=nexsindx)
                 [(await wind.put((offs, changes))) for wind in tuple(self.windows)]
 
         await asyncio.sleep(0)
 
         return results
 
-    async def _storNodeEdit(self, nodeedit):
+    async def _storNodeEdit(self, nodeedit, meta):
         '''
         Execute a series of storage operations for the given node.
 
         Args:
             nodeedit
+            meta
 
         Returns a list of flattened NodeEdits (Tuple[BuidT, str, Sequence]), with top-level and subedits all at the
         top level.  The 3rd element of each item contains only the applied changes.
@@ -1375,9 +1508,21 @@ class Layer(s_nexus.Pusher):
             postedits.extend(changes)
 
             if changes:
+                if any((c[0] == EDIT_NODE_ADD for c in changes)):
+                    if sode is not None:
+                        sode['ndef'] = (form, edit[1][0])
+
+                    if meta is not None:
+                        ctime = meta.get('time')
+                    else:
+                        ctime = s_common.now()
+
+                    created = (EDIT_PROP_SET, ('.created', ctime, None, STOR_TYPE_MINTIME))
+                    postedits.extend(self._editPropSet(buid, form, created, sode))
+
                 subedits = edit[2]
                 for ne in subedits:
-                    subpostnodeedits = await self._storNodeEdit(ne)
+                    subpostnodeedits = await self._storNodeEdit(ne, meta)
                     # Consolidate any changes from subedits to the main edit node into that nodeedit's
                     # edits, otherwise just append an entire nodeedit
                     for subpostnodeedit in subpostnodeedits:
@@ -1436,12 +1581,6 @@ class Layer(s_nexus.Pusher):
 
         retn = [(EDIT_NODE_ADD, (valu, stortype), ())]
 
-        if sode is not None:
-            sode['ndef'] = (form, valu)
-
-        created = (EDIT_PROP_SET, ('.created', s_common.now(), None, STOR_TYPE_MINTIME))
-        retn.extend(self._editPropSet(buid, form, created, sode))
-
         return retn
 
     def _editNodeDel(self, buid, form, edit, sode):
@@ -2100,7 +2239,7 @@ class Layer(s_nexus.Pusher):
 
                     iden = await proxy.getIden()
                     offs = self.offsets.get(iden)
-                    logger.warning(f'upstream sync connected ({url} offset={offs})')
+                    logger.warning(f'upstream sync connected ({s_urlhelp.sanitizeUrl(url)} offset={offs})')
 
                     if offs == 0:
                         offs = await proxy.getNodeEditOffset()
@@ -2199,7 +2338,7 @@ class Layer(s_nexus.Pusher):
         if size is not None:
 
             count = 0
-            for offset, (nodeedits, meta) in self.nodeeditlog.iter(offs[0]):
+            async for offset, nodeedits, meta in self.iterNodeEditLog(offs[0]):
                 async for splice in self.makeSplices(offset, nodeedits, meta):
 
                     if splice[0] < offs:
@@ -2211,7 +2350,7 @@ class Layer(s_nexus.Pusher):
                     yield splice
                     count = count + 1
         else:
-            for offset, (nodeedits, meta) in self.nodeeditlog.iter(offs[0]):
+            async for offset, nodeedits, meta in self.iterNodeEditLog(offs[0]):
                 async for splice in self.makeSplices(offset, nodeedits, meta):
 
                     if splice[0] < offs:
@@ -2225,12 +2364,12 @@ class Layer(s_nexus.Pusher):
             return
 
         if offs is None:
-            offs = (self.nodeeditlog.index(), 0, 0)
+            offs = (await self.getNodeEditOffset(), 0, 0)
 
         if size is not None:
 
             count = 0
-            for offset, (nodeedits, meta) in self.nodeeditlog.iterBack(offs[0]):
+            async for offset, nodeedits, meta in self.iterNodeEditLogBack(offs[0]):
                 async for splice in self.makeSplices(offset, nodeedits, meta, reverse=True):
 
                     if splice[0] > offs:
@@ -2242,7 +2381,7 @@ class Layer(s_nexus.Pusher):
                     yield splice
                     count += 1
         else:
-            for offset, (nodeedits, meta) in self.nodeeditlog.iterBack(offs[0]):
+            async for offset, nodeedits, meta in self.iterNodeEditLogBack(offs[0]):
                 async for splice in self.makeSplices(offset, nodeedits, meta, reverse=True):
 
                     if splice[0] > offs:
@@ -2250,6 +2389,20 @@ class Layer(s_nexus.Pusher):
 
                     yield splice
 
+    async def iterNodeEditLog(self, offs=0):
+        '''
+        Iterate the node edit log and yield (offs, edits, meta) tuples.
+        '''
+        for offs, (edits, meta) in self.nodeeditlog.iter(offs):
+            yield (offs, edits, meta)
+
+    async def iterNodeEditLogBack(self, offs=0):
+        '''
+        Iterate the node edit log and yield (offs, edits, meta) tuples in reverse.
+        '''
+        for offs, (edits, meta) in self.nodeeditlog.iterBack(offs):
+            yield (offs, edits, meta)
+
     async def syncNodeEdits(self, offs, wait=True):
         '''
         Yield (offs, nodeedits) tuples from the nodeedit log starting from the given offset.
@@ -2416,3 +2569,20 @@ class Layer(s_nexus.Pusher):
         '''
         await self.fini()
         shutil.rmtree(self.dirn, ignore_errors=True)
+
+def getFlatEdits(nodeedits):
+
+    editsbynode = collections.defaultdict(list)
+
+    # flatten out conditional node edits
+    def addedits(buid, form, edits):
+        nkey = (buid, form)
+        for edittype, editinfo, condedits in edits:
+            editsbynode[nkey].append((edittype, editinfo, ()))
+            for condedit in condedits:
+                addedits(*condedit)
+
+    for buid, form, edits in nodeedits:
+        addedits(buid, form, edits)
+
+    return [(k[0], k[1], v) for (k, v) in editsbynode.items()]
