@@ -6,7 +6,6 @@ import collections
 import synapse.exc as s_exc
 import synapse.common as s_common
 
-import synapse.lib.ast as s_ast
 import synapse.lib.coro as s_coro
 import synapse.lib.snap as s_snap
 import synapse.lib.nexus as s_nexus
@@ -24,9 +23,13 @@ reqValidVdef = s_config.getJsValidator({
         'name': {'type': 'string'},
         'parent': {'type': ['string', 'null'], 'pattern': s_config.re_iden},
         'creator': {'type': 'string', 'pattern': s_config.re_iden},
+        'layers': {
+            'type': 'array',
+            'items': {'type': 'string', 'pattern': s_config.re_iden}
+        },
     },
     'additionalProperties': True,
-    'required': ['iden', 'parent', 'creator'],
+    'required': ['iden', 'parent', 'creator', 'layers'],
 })
 
 class View(s_nexus.Pusher):  # type: ignore
@@ -161,6 +164,8 @@ class View(s_nexus.Pusher):  # type: ignore
         opts = self.core._initStormOpts(opts)
         user = self.core._userFromOpts(opts)
 
+        self.core._logStormQuery(text, user)
+
         info = {'query': text, 'opts': opts}
         await self.core.boss.promote('storm', user=user, info=info)
 
