@@ -3,21 +3,16 @@ import synapse.exc as s_exc
 import synapse.lib.types as s_types
 import synapse.lib.module as s_module
 
-class DnsName(s_types.StrBase):
+class DnsName(s_types.Str):
 
     def postTypeInit(self):
 
-        s_types.StrBase.postTypeInit(self)
+        s_types.Str.postTypeInit(self)
         self.inarpa = '.in-addr.arpa'
         self.inarpa6 = '.ip6.arpa'
 
         self.setNormFunc(str, self._normPyStr)
 
-    def indxByPref(self, valu):
-        valu = valu.lower()
-        valu = valu.strip()
-        return s_types.StrBase.indxByPref(self, valu)
-
     def _normPyStr(self, valu):
         # Backwards compatible
         norm = valu.lower()
@@ -266,10 +261,10 @@ class DnsModule(s_module.CoreModule):
                         'doc': 'The DNS AAAA record returned by the lookup.',
                     }),
                     ('rev6', ('inet:dns:rev6', {}), {'ro': True,
-                        'doc': 'The DNS PTR record returned by the lookup of a IPv6 address.',
+                        'doc': 'The DNS PTR record returned by the lookup of an IPv6 address.',
                     }),
                     ('cname', ('inet:dns:cname', {}), {'ro': True,
-                        'doc': 'The DNS CNAME record returned by the lookup',
+                        'doc': 'The DNS CNAME record returned by the lookup.',
                     }),
                     ('mx', ('inet:dns:mx', {}), {'ro': True,
                         'doc': 'The DNS MX record returned by the lookup.',
