@@ -8,7 +8,7 @@ import synapse.lib.version as s_version
 
 logger = logging.getLogger(__name__)
 
-class SemVer(s_types.Type):
+class SemVer(s_types.Int):
     '''
     Provides support for parsing a semantic version string into its component
     parts. This normalizes a version string into an integer to allow version
@@ -24,6 +24,7 @@ class SemVer(s_types.Type):
     strings if that information is present.
     '''
     def postTypeInit(self):
+        s_types.Int.postTypeInit(self)
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(int, self._normPyInt)
 
@@ -59,9 +60,6 @@ class SemVer(s_types.Type):
         valu = s_version.fmtVersion(major, minor, patch)
         return valu
 
-    def indx(self, valu):
-        return valu.to_bytes(8, 'big')
-
 class ItModule(s_module.CoreModule):
     async def initCoreModule(self):
         self.model.form('it:dev:str').onAdd(self._onFormItDevStr)
@@ -113,9 +111,10 @@ class ItModule(s_module.CoreModule):
         # Check to see if name is available and set it if possible
         prop = node.get('software')
         if prop:
-            snodes = [n async for n in node.snap.getNodesBy('it:prod:soft', prop)]
-            if snodes:
-                name = snodes[0].get('name')
+            opts = {'vars': {'soft': prop}}
+            nodes = await node.snap.nodes('it:prod:soft=$soft', opts=opts)
+            if nodes:
+                name = nodes[0].get('name')
                 if name:
                     await node.set('software:name', name)
 
@@ -151,12 +150,12 @@ class ItModule(s_module.CoreModule):
         modl = {
             'ctors': (
                 ('it:semver', 'synapse.models.infotech.SemVer', {}, {
-                    'doc': 'Semantic Version type',
+                    'doc': 'Semantic Version type.',
                 }),
             ),
             'types': (
                 ('it:hostname', ('str', {'strip': True, 'lower': True}), {
-                    'doc': 'The name of a host or sytsem',
+                    'doc': 'The name of a host or system.',
                 }),
                 ('it:host', ('guid', {}), {
                     'doc': 'A GUID that represents a host or system.'
@@ -248,7 +247,7 @@ class ItModule(s_module.CoreModule):
                     'doc': 'A file that triggered an alert on a specific antivirus signature.',
                 }),
                 ('it:auth:passwdhash', ('guid', {}), {
-                    'doc': 'An instance of a password hash',
+                    'doc': 'An instance of a password hash.',
                 }),
                 ('it:exec:proc', ('guid', {}), {
                     'doc': 'A process executing on a host. May be an actual (e.g., endpoint) or virtual (e.g., malware sandbox) host.',
@@ -260,7 +259,7 @@ class ItModule(s_module.CoreModule):
                     'doc': 'A named pipe created by a process at runtime.',
                 }),
                 ('it:exec:url', ('guid', {}), {
-                    'doc': 'A instance of a host requesting a URL.',
+                    'doc': 'An instance of a host requesting a URL.',
                 }),
                 ('it:exec:bind', ('guid', {}), {
                     'doc': 'An instance of a host binding a listening port.',
@@ -284,7 +283,7 @@ class ItModule(s_module.CoreModule):
                     'doc': 'An instance of a host getting a registry key.',
                 }),
                 ('it:exec:reg:set', ('guid', {}), {
-                    'doc': 'An instance of a host creating or setting a registry key',
+                    'doc': 'An instance of a host creating or setting a registry key.',
                 }),
                 ('it:exec:reg:del', ('guid', {}), {
                     'doc': 'An instance of a host deleting a registry key.',
@@ -310,7 +309,9 @@ class ItModule(s_module.CoreModule):
                 ('it:reveng:funcstr', ('comp', {'fields': (('function', 'it:reveng:function'), ('string', 'str'))}), {
                     'doc': 'A reference to a string inside a function.',
                 }),
-
+                ('it:reveng:impfunc', ('str', {'lower': 1}), {
+                    'doc': 'A function from an imported library.',
+                }),
             ),
             'forms': (
                 ('it:hostname', {}, ()),
@@ -322,7 +323,6 @@ class ItModule(s_module.CoreModule):
                     ('desc', ('str', {}), {
                         'doc': 'A free-form description of the host.',
                     }),
-                    #  FIXME we probably eventually need a bunch of stuff here...
                     ('ipv4', ('inet:ipv4', {}), {
                         'doc': 'The last known ipv4 address for the host.'
                     }),
@@ -398,7 +398,7 @@ class ItModule(s_module.CoreModule):
                         'doc': 'A short description of the software.',
                     }),
                     ('author:org', ('ou:org', {}), {
-                        'doc': 'Organization which authored the software',
+                        'doc': 'Organization which authored the software.',
                     }),
                     ('author:acct', ('inet:web:acct', {}), {
                         'doc': 'Web account of the software author.',
@@ -466,7 +466,7 @@ class ItModule(s_module.CoreModule):
                 ('it:prod:softver', {}, (
 
                     ('software', ('it:prod:soft', {}), {
-                        'doc': 'Software associated with this version instance',
+                        'doc': 'Software associated with this version instance.',
                     }),
                     ('software:name', ('str', {'lower': True, 'strip': True}), {
                         'doc': 'The name of the software at a particular version.',
@@ -478,7 +478,7 @@ class ItModule(s_module.CoreModule):
                         'doc': 'Normalized version of the version string.',
                     }),
                     ('arch', ('it:dev:str', {}), {
-                        'doc': 'Software architecture',
+                        'doc': 'Software architecture.',
                     }),
                     ('semver', ('it:semver', {}), {
                         'doc': 'System normalized semantic version number.',
@@ -540,7 +540,7 @@ class ItModule(s_module.CoreModule):
                         'doc': 'The signature name.'
                     }),
                     ('desc', ('str', {}), {
-                        'doc': 'A free-form description of the signature',
+                        'doc': 'A free-form description of the signature.',
                     }),
                     ('url', ('inet:url', {}), {
                         'doc': 'A reference URL for information about the signature.',
@@ -602,7 +602,7 @@ class ItModule(s_module.CoreModule):
                         'doc': 'The command string used to launch the process, including any command line parameters.',
                     }),
                     ('pid', ('int', {}), {
-                        'doc': 'The process ID',
+                        'doc': 'The process ID.',
                     }),
                     ('time', ('time', {}), {
                         'doc': 'The start time for the process.',
@@ -614,7 +614,7 @@ class ItModule(s_module.CoreModule):
                         'doc': 'The path to the executable of the process.',
                     }),
                     ('src:exe', ('file:path', {}), {
-                        'doc': 'The path to the executable which started the process',
+                        'doc': 'The path to the executable which started the process.',
                     }),
                     ('src:proc', ('it:exec:proc', {}), {
                         'doc': 'The process which created the process.'
@@ -966,6 +966,8 @@ class ItModule(s_module.CoreModule):
                         'doc': 'The yara rule text.'}),
                     ('name', ('str', {}), {
                         'doc': 'The name of the yara rule.'}),
+                    ('author', ('ps:contact', {}), {
+                        'doc': 'Contact info for the author of the yara rule.'}),
                     ('version', ('it:semver', {}), {
                         'doc': 'The current version of the rule.'}),
                 )),
@@ -983,31 +985,43 @@ class ItModule(s_module.CoreModule):
 
                 ('it:reveng:function', {}, (
                     ('name', ('str', {}), {
-                        'doc': 'The name of the function'}),
+                        'doc': 'The name of the function.'}),
                     ('description', ('str', {}), {
-                        'doc': 'Notes concerning the function'}),
+                        'doc': 'Notes concerning the function.'}),
+                    ('impcalls', ('array', {'type': 'it:reveng:impfunc'}), {
+                        'doc': 'Calls to imported library functions within the scope of the function.',
+                    }),
                 )),
 
                 ('it:reveng:filefunc', {}, (
                     ('function', ('it:reveng:function', {}), {
                         'ro': True,
-                        'doc': 'The guid matching the function'}),
+                        'doc': 'The guid matching the function.'}),
                     ('file', ('file:bytes', {}), {
                         'ro': True,
                         'doc': 'The file that contains the function.'}),
                     ('va', ('int', {}), {
-                        'doc': 'The virtual address of the first codeblock of the function'}),
+                        'doc': 'The virtual address of the first codeblock of the function.'}),
+                    ('rank', ('int', {}), {
+                        'doc': 'The function rank score used to evaluate if it exhibits interesting behavior.'}),
+                    ('complexity', ('int', {}), {
+                        'doc': 'The complexity of the function.'}),
+                    ('funccalls', ('array', {'type': 'it:reveng:filefunc'}), {
+                        'doc': 'Other function calls within the scope of the function.',
+                    }),
                 )),
 
                 ('it:reveng:funcstr', {}, (
                     ('function', ('it:reveng:function', {}), {
                         'ro': True,
-                        'doc': 'The guid matching the function'}),
+                        'doc': 'The guid matching the function.'}),
                     ('string', ('str', {}), {
                         'ro': True,
                         'doc': 'The string that the function references.'}),
                 )),
 
+                ('it:reveng:impfunc', {}, ()),
+
             ),
         }
         name = 'it'
