@@ -20,8 +20,11 @@ def chop_imei(imei):
     cd = int(imei[14:15])
     return valu, {'subs': {'tac': tac, 'serial': snr, 'cd': cd}}
 
-class Phone(s_types.Type):
+class Phone(s_types.Str):
+
     def postTypeInit(self):
+        s_types.Str.postTypeInit(self)
+        self.opts['globsuffix'] = True
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(int, self._normPyInt)
 
@@ -48,30 +51,10 @@ class Phone(s_types.Type):
                                     mesg='phone int must be greater than 0')
         return self._normPyStr(str(valu))
 
-    def indx(self, valu):
-        '''
-
-        Args:
-            valu (str): Value to encode
-
-        Returns:
-            bytes: Encoded value
-        '''
-        return valu.encode('utf8')
-
-    def indxByEq(self, valu):
-        if isinstance(valu, str) and valu.endswith('*'):
-            norm, _ = self._normPyStr(valu)
-            return (
-                ('pref', self.indx(norm)),
-            )
-        return s_types.Type.indxByEq(self, valu)
-
     def repr(self, valu):
-        # FIXME implement more geo aware reprs
         # XXX geo-aware reprs are practically a function of cc which
         # XXX the raw value may only have after doing a s_l_phone lookup
-        if valu[0] == '1':  # FIXME Length check
+        if valu[0] == '1' and len(valu) == 11:
             area = valu[1:4]
             pref = valu[4:7]
             numb = valu[7:11]
@@ -100,17 +83,12 @@ def imeicsum(text):
 
     return str(chek)
 
-class Imsi(s_types.Type):
-    def postTypeInit(self):
-        self.setNormFunc(str, self._normPyStr)
-        self.setNormFunc(int, self._normPyInt)
+class Imsi(s_types.Int):
 
-    def _normPyStr(self, valu):
-        digs = digits(valu)
-        if not digs:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg='requires a digit string')
-        return self._normPyInt(int(digs))
+    def postTypeInit(self):
+        self.opts['size'] = 8
+        self.opts['signed'] = False
+        return s_types.Int.postTypeInit(self)
 
     def _normPyInt(self, valu):
         imsi = str(valu)
@@ -123,29 +101,13 @@ class Imsi(s_types.Type):
         # TODO full imsi analysis tree
         return valu, {'subs': {'mcc': mcc}}
 
-    def indx(self, valu):
-        '''
-
-        Args:
-            valu (int):
-
-        Returns:
-            bytes:
-        '''
-        return valu.to_bytes(8, byteorder='big')
-
 # TODO: support pre 2004 "old" imei format
-class Imei(s_types.Type):
-    def postTypeInit(self):
-        self.setNormFunc(str, self._normPyStr)
-        self.setNormFunc(int, self._normPyInt)
+class Imei(s_types.Int):
 
-    def _normPyStr(self, valu):
-        digs = digits(valu)
-        if not digs:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg='requires a digit string')
-        return self._normPyInt(int(digs))
+    def postTypeInit(self):
+        self.opts['size'] = 8
+        self.opts['signed'] = False
+        return s_types.Int.postTypeInit(self)
 
     def _normPyInt(self, valu):
         imei = str(valu)
@@ -167,17 +129,6 @@ class Imei(s_types.Type):
         raise s_exc.BadTypeValu(valu=valu, name=self.name,
                                 mesg='Failed to norm IMEI')
 
-    def indx(self, valu):
-        '''
-
-        Args:
-            valu (int):
-
-        Returns:
-            bytes:
-        '''
-        return valu.to_bytes(7, byteorder='big')
-
 class TelcoModule(s_module.CoreModule):
     def getModelDefs(self):
         modl = {
@@ -185,11 +136,11 @@ class TelcoModule(s_module.CoreModule):
 
                 ('tel:mob:imei', 'synapse.models.telco.Imei', {}, {
                     'ex': '490154203237518',
-                    'doc': 'An International Mobile Equipment Id'}),
+                    'doc': 'An International Mobile Equipment Id.'}),
 
                 ('tel:mob:imsi', 'synapse.models.telco.Imsi', {}, {
                     'ex': '310150123456789',
-                    'doc': 'An International Mobile Subscriber Id'}),
+                    'doc': 'An International Mobile Subscriber Id.'}),
 
                 ('tel:phone', 'synapse.models.telco.Phone', {}, {
                     'ex': '+15558675309',
@@ -207,7 +158,7 @@ class TelcoModule(s_module.CoreModule):
 
                 ('tel:mob:tac', ('int', {}), {
                     'ex': '49015420',
-                    'doc': 'A mobile Type Allocation Code'}),
+                    'doc': 'A mobile Type Allocation Code.'}),
 
                 ('tel:mob:imid', ('comp', {'fields': (('imei', 'tel:mob:imei'), ('imsi', 'tel:mob:imsi'))}), {
                     'ex': '(490154203237518, 310150123456789)',
@@ -221,11 +172,11 @@ class TelcoModule(s_module.CoreModule):
                     'doc': 'A single mobile telemetry measurement.'}),
 
                 ('tel:mob:mcc', ('str', {'regex': '^[0-9]{3}$', 'strip': 1}), {
-                    'doc': 'ITU Mobile Country Code',
+                    'doc': 'ITU Mobile Country Code.',
                 }),
 
                 ('tel:mob:mnc', ('str', {'regex': '^[0-9]{2,3}$', 'strip': 1}), {
-                    'doc': 'ITU Mobile Network Code',
+                    'doc': 'ITU Mobile Network Code.',
                 }),
 
                 ('tel:mob:carrier', ('comp', {'fields': (('mcc', 'tel:mob:mcc'), ('mnc', 'tel:mob:mnc'))}), {
@@ -244,7 +195,6 @@ class TelcoModule(s_module.CoreModule):
                 ('tel:phone', {}, (
                     ('loc', ('loc', {}), {
                         'doc': 'The location associated with the number.',
-                        'defval': '??',
                     }),
                 )),
                 ('tel:call', {}, (
@@ -287,7 +237,7 @@ class TelcoModule(s_module.CoreModule):
                         'doc': 'The time the message was sent.'
                     }),
                     ('text', ('str', {}), {
-                        'doc': 'The text of the message',
+                        'doc': 'The text of the message.',
                     }),
                     ('file', ('file:bytes', {}), {
                         'doc': 'A file containing related media.',
@@ -295,29 +245,26 @@ class TelcoModule(s_module.CoreModule):
                 )),
                 ('tel:mob:tac', {}, (
                     ('org', ('ou:org', {}), {
-                        'doc': 'The org guid for the manufacturer',
+                        'doc': 'The org guid for the manufacturer.',
                     }),
                     ('manu', ('str', {'lower': 1}), {
-                        'doc': 'The TAC manufacturer name',
-                        'defval': '??',
+                        'doc': 'The TAC manufacturer name.',
                     }),
                     ('model', ('str', {'lower': 1}), {
-                        'doc': 'The TAC model name',
-                        'defval': '??',
+                        'doc': 'The TAC model name.',
                     }),
                     ('internal', ('str', {'lower': 1}), {
-                        'doc': 'The TAC internal model name',
-                        'defval': '??',
+                        'doc': 'The TAC internal model name.',
                     }),
                 )),
                 ('tel:mob:imei', {}, (
                     ('tac', ('tel:mob:tac', {}), {
                         'ro': 1,
-                        'doc': 'The Type Allocate Code within the IMEI'
+                        'doc': 'The Type Allocate Code within the IMEI.'
                     }),
                     ('serial', ('int', {}), {
                         'ro': 1,
-                        'doc': 'The serial number within the IMEI',
+                        'doc': 'The serial number within the IMEI.',
                     })
                 )),
                 ('tel:mob:imsi', {}, (
@@ -363,11 +310,11 @@ class TelcoModule(s_module.CoreModule):
                     }),
                 )),
                 ('tel:mob:cell', {}, (
-                    ('carrier', ('tel:mob:carrier', {}), {'doc': 'Mobile carrier'}),
-                    ('carrier:mcc', ('tel:mob:mcc', {}), {'doc': 'Mobile Country Code'}),
-                    ('carrier:mnc', ('tel:mob:mnc', {}), {'doc': 'Mobile Network Code'}),
+                    ('carrier', ('tel:mob:carrier', {}), {'doc': 'Mobile carrier.'}),
+                    ('carrier:mcc', ('tel:mob:mcc', {}), {'doc': 'Mobile Country Code.'}),
+                    ('carrier:mnc', ('tel:mob:mnc', {}), {'doc': 'Mobile Network Code.'}),
                     ('lac', ('int', {}), {'doc': 'Location Area Code. LTE networks may call this a TAC.'}),
-                    ('cid', ('int', {}), {'doc': 'Cell ID'}),
+                    ('cid', ('int', {}), {'doc': 'The Cell ID.'}),
                     ('radio', ('str', {'lower': 1, 'onespace': 1}), {'doc': 'Cell radio type.'}),
                     ('latlong', ('geo:latlong', {}), {'doc': 'Last known location of the cell site.'}),
 
@@ -383,6 +330,9 @@ class TelcoModule(s_module.CoreModule):
                     ('time', ('time', {}), {}),
                     ('latlong', ('geo:latlong', {}), {}),
 
+                    ('host', ('it:host', {}), {
+                        'doc': 'The host that generated the mobile telemetry data.'}),
+
                     ('place', ('geo:place', {}), {
                         'doc': 'The place representing the location of the mobile telemetry sample.'}),
 
