@@ -626,6 +626,7 @@ class Client(s_base.Base):
         self._t_proxy = None
         self._t_ready = asyncio.Event()
         self._t_onlink = onlink
+        self._t_methinfo = None
 
         async def fini():
             if self._t_proxy is not None:
@@ -676,14 +677,19 @@ class Client(s_base.Base):
                 raise
 
             except Exception as e:
-                logger.warning(f'telepath client ({url}): {e}')
+                logger.warning(f'telepath client ({s_urlhelp.sanitizeUrl(url)}): {e}')
                 await self.waitfini(timeout=self._t_conf.get('retrysleep', 0.2))
 
+    async def proxy(self, timeout=10):
+        await self.waitready(timeout=timeout)
+        return self._t_proxy
+
     async def _initTeleLink(self, url):
         if self._t_proxy is not None:
             await self._t_proxy.fini()
 
         self._t_proxy = await openurl(url, **self._t_opts)
+        self._t_methinfo = self._t_proxy.methinfo
 
         async def fini():
             await self._fireLinkLoop()
@@ -712,15 +718,17 @@ class Client(s_base.Base):
             except s_exc.TeleRedir as e:
                 url = e.errinfo.get('url')
                 self._setNextUrl(url)
-                logger.warning(f'telepath task redirected: ({url})')
+                logger.warning(f'telepath task redirected: ({s_urlhelp.sanitizeUrl(url)})')
                 await self._t_proxy.fini()
 
-    async def waitready(self):
-        await asyncio.wait_for(self._t_ready.wait(), self._t_conf.get('timeout', 10))
+    async def waitready(self, timeout=10):
+        await asyncio.wait_for(self._t_ready.wait(), self._t_conf.get('timeout', timeout))
 
     def __getattr__(self, name):
+        if self._t_methinfo is None:
+            raise s_exc.NotReady(mesg='Must call waitready() on Client before first method call')
 
-        info = self._t_proxy.methinfo.get(name)
+        info = self._t_methinfo.get(name)
         if info is not None and info.get('genr'):
             meth = GenrMethod(self, name)
             setattr(self, name, meth)
@@ -771,7 +779,7 @@ async def disc_consul(info):
 
         The following HTTP parameters are supported:
 
-        - consul: This is the consul host (schema, fqdn and port) to connect too.
+        - consul: This is the consul host (schema, fqdn and port) to connect to.
         - consul_tag: If set, iterate through the catalog results until a result
           is found which matches the tag value. This is a case sensitive match.
         - consul_tag_address: If set, prefer the ``TaggedAddresses`` from the catalog.
@@ -954,7 +962,10 @@ async def openurl(url, **opts):
 
     elif scheme == 'unix':
         # unix:///path/to/sock:share
-        path, name = info.get('path').split(':')
+        name = '*'
+        path = info.get('path')
+        if ':' in path:
+            path, name = path.split(':')
         link = await s_link.unixconnect(path)
 
     else:
@@ -964,9 +975,21 @@ async def openurl(url, **opts):
 
         sslctx = None
         if scheme == 'ssl':
-            certpath = info.get('certdir')
-            certdir = s_certdir.CertDir(certpath)
-            sslctx = certdir.getClientSSLContext()
+
+            certname = None
+            certpath = None
+
+            certdir = opts.get('certdir')
+
+            query = info.get('query')
+            if query is not None:
+                certpath = query.get('certdir')
+                certname = query.get('certname')
+
+            if certdir is None:
+                certdir = s_certdir.CertDir(certpath)
+
+            sslctx = certdir.getClientSSLContext(certname=certname)
 
         link = await s_link.connect(host, port, ssl=sslctx)
 
