@@ -298,7 +298,7 @@ class NodeTest(s_t_utils.SynTest):
                 # self.none(path.getVar('bar'))
                 self.eq(path.getVar('bar'), 'ru')
                 self.eq(path.getVar('key'), 'valu')
-                path.finiframe()
+                path.finiframe(path.runt)
                 self.len(0, path.frames)
                 self.eq(path.getVar('bar'), 'ru')
                 self.eq(path.getVar('key'), s_common.novalu)
@@ -310,7 +310,7 @@ class NodeTest(s_t_utils.SynTest):
                 path.initframe({'key': 'valu'}, initrunt='runt')
                 self.eq(path.runt, 'runt')
                 self.true(path.frames[0][1] is runt)
-                path.finiframe()
+                path.finiframe(runt)
                 self.true(path.runt is runt)
 
                 # Path clone() creates a fully independent Path object
@@ -332,14 +332,22 @@ class NodeTest(s_t_utils.SynTest):
                 self.len(1, pcln.frames)
                 self.eq(path.getVar('key'), 'valu')
                 self.eq(pcln.getVar('key'), 'valu')
-                pcln.finiframe()
-                path.finiframe()
+                pcln.finiframe(pcln.runt)
+                path.finiframe(path.runt)
                 pcln.setVar('bar', 'us')
                 self.eq(pcln.getVar('bar'), 'us')
                 self.eq(path.getVar('bar'), 'ru')
                 self.eq(pcln.getVar('key'), s_common.novalu)
                 self.eq(path.getVar('key'), s_common.novalu)
 
+                # Check that finiframe without frames resets vars and has correct runt
+                with snap.getStormRuntime() as subr:
+                    node = await snap.addNode('test:comp', (35, 'cat'))
+                    nodepaths = await alist(node.storm('-> test:int [:loc=$foo]', opts={'vars': {'foo': 'ca'}}))
+                    path = nodepaths[0][1]
+                    path.finiframe(subr)
+                    self.eq(path.getVar('foo'), s_common.novalu)
+
         # Ensure that path clone() behavior in storm is as expected
         # with a real-world style test..
         async with self.getTestCore() as core:
@@ -350,7 +358,7 @@ class NodeTest(s_t_utils.SynTest):
             $x = $( $x + 1 )
             $lib.fire(test, valu=$node.value(), x=$x)
             -test:int'''
-            msgs = await core.streamstorm(q).list()
+            msgs = await core.stormlist(q)
             data = collections.defaultdict(set)
             for m in msgs:
                 if m[0] == 'storm:fire':
@@ -375,5 +383,38 @@ class NodeTest(s_t_utils.SynTest):
             with self.raises(s_exc.NoSuchProp):
                 node.repr('newp')
 
-            with self.raises(s_exc.NoPropValu):
-                node.repr('dns:rev')
+            self.none(node.repr('dns:rev'))
+
+    async def test_node_data(self):
+        async with self.getTestCore() as core:
+            nodes = await core.nodes('[ inet:ipv4=1.2.3.4 :loc=us ]')
+            self.len(1, nodes)
+
+            node = nodes[0]
+
+            self.none(await node.getData('foo'))
+
+            await node.setData('foo', 123)
+            self.eq(123, await node.getData('foo'))
+
+            await node.setData('foo', 123)
+            self.eq(123, await node.getData('foo'))
+
+            await node.setData('bar', (4, 5, 6))
+            self.eq((4, 5, 6), await node.getData('bar'))
+            self.eq(123, await node.getData('foo'))
+
+            self.eq([('foo', 123), ('bar', (4, 5, 6))], await alist(node.iterData()))
+
+            self.eq(123, await node.popData('foo'))
+            self.none(await node.getData('foo'))
+            self.none(await node.popData('foo'))
+
+            self.eq((4, 5, 6), await node.getData('bar'))
+
+            await node.delete()
+            nodes = await core.nodes('[ inet:ipv4=1.2.3.4 :loc=us ]')
+            node = nodes[0]
+
+            self.none(await node.getData('foo'))
+            self.none(await node.getData('bar'))
