@@ -18,6 +18,10 @@ class StormTest(s_t_utils.SynTest):
             vals = [n.ndef[1] for n in nodes]
             self.eq(('www.vertex.link', 'vertex.link', 'link'), vals)
 
+            # Max recursion fail
+            q = '[ inet:fqdn=www.vertex.link ] | tree { inet:fqdn=www.vertex.link }'
+            await self.asyncraises(s_exc.StormRuntimeError, core.nodes(q))
+
     async def test_storm_movetag(self):
 
         async with self.getTestCore() as core:
@@ -104,7 +108,6 @@ class StormTest(s_t_utils.SynTest):
 
         # Test moving a tag to another tag which is a string prefix of the source
         async with self.getTestCore() as core:
-            # core.conf['storm:log'] = True
             async with await core.snap() as snap:
                 node = await snap.addNode('test:str', 'V')
                 await node.addTag('aaa.b.ccc', (None, None))
@@ -276,12 +279,12 @@ class StormTest(s_t_utils.SynTest):
 
             # Universal prop for relative path
             nodes = await core.nodes('.created>=$minc | max .created',
-                                    {'vars': {'minc': minc}})
+                                     {'vars': {'minc': minc}})
             self.len(1, nodes)
             self.eq(nodes[0].get('tick'), midval)
 
             nodes = await core.nodes('.created>=$minc | min .created',
-                                    {'vars': {'minc': minc}})
+                                     {'vars': {'minc': minc}})
             self.len(1, nodes)
             self.eq(nodes[0].get('tick'), minval)
 
@@ -323,7 +326,7 @@ class StormTest(s_t_utils.SynTest):
             maxnodes = await core.nodes('[ ou:org=* ]')
             maxnodes = await core.nodes('[ ou:org=* +#minmax ]')
             minnodes = await core.nodes('[ ou:org=* +#minmax=(1981, 2010) ]')
-            midnodes = await core.nodes('[ ou:org=* +#minmax=(1982, 2018) ]')
+            await core.nodes('[ ou:org=* +#minmax=(1982, 2018) ]')
             maxnodes = await core.nodes('[ ou:org=* +#minmax=(1997, 2020) ]')
 
             testmin = await core.nodes('ou:org | min #minmax')
@@ -370,9 +373,9 @@ class StormTest(s_t_utils.SynTest):
             self.len(1, nodes)
             self.eq(nodes[0].ndef, ('inet:ipv4', 0x05050505))
 
-            nodes = await core.nodes('inet:search:query | scrape :text --refs --yield')
+            nodes = await core.nodes('inet:search:query | scrape :text --refs | -(refs)> *')
             self.len(1, nodes)
-            self.eq(nodes[0].ndef, ('edge:refs', (('inet:search:query', guid), ('inet:ipv4', 0x05050505))))
+            self.eq(nodes[0].ndef, ('inet:ipv4', 0x05050505))
 
             # test runtsafe and non-runtsafe failure to create node
             msgs = await core.stormlist('scrape "https://t.c…"')
