@@ -6,7 +6,53 @@ from synapse.tests.utils import alist
 class OuModelTest(s_t_utils.SynTest):
 
     async def test_ou_simple(self):
+
         async with self.getTestCore() as core:
+
+            goal = s_common.guid()
+            org0 = s_common.guid()
+            camp = s_common.guid()
+
+            async with await core.snap() as snap:
+
+                props = {
+                    'name': 'MyGoal',
+                    'type': 'MyType',
+                    'desc': 'MyDesc',
+                    'prev': goal,
+                }
+                node = await snap.addNode('ou:goal', goal, props=props)
+                self.eq(node.get('name'), 'MyGoal')
+                self.eq(node.get('type'), 'MyType')
+                self.eq(node.get('desc'), 'MyDesc')
+                self.eq(node.get('prev'), goal)
+
+                props = {
+                    'stated': True,
+                    'window': '2019,2020',
+                }
+                node = await snap.addNode('ou:hasgoal', (org0, goal), props=props)
+                self.eq(node.get('org'), org0)
+                self.eq(node.get('goal'), goal)
+                self.eq(node.get('stated'), True)
+                self.eq(node.get('window'), (1546300800000, 1577836800000))
+
+                props = {
+                    'org': org0,
+                    'goal': goal,
+                    'goals': (goal,),
+                    'name': 'MyName',
+                    'type': 'MyType',
+                    'desc': 'MyDesc',
+                }
+                node = await snap.addNode('ou:campaign', camp, props=props)
+                self.eq(node.get('org'), org0)
+                self.eq(node.get('goal'), goal)
+                self.eq(node.get('goals'), (goal,))
+                self.eq(node.get('name'), 'MyName')
+                self.eq(node.get('type'), 'MyType')
+                self.eq(node.get('desc'), 'MyDesc')
+
             # type norming first
             # ou:name
             t = core.model.type('ou:name')
@@ -41,6 +87,45 @@ class OuModelTest(s_t_utils.SynTest):
             self.eq(t.norm('HAHA1')[0], 'haha1')
             self.eq(t.norm('GOV_MFA')[0], 'gov_mfa')
 
+            # ou:position / ou:org:subs
+            orgiden = s_common.guid()
+            contact = s_common.guid()
+            position = s_common.guid()
+            subpos = s_common.guid()
+            suborg = s_common.guid()
+
+            opts = {'vars': {
+                'orgiden': orgiden,
+                'contact': contact,
+                'position': position,
+                'subpos': subpos,
+                'suborg': suborg,
+            }}
+
+            nodes = await core.nodes('''
+                [ ou:org=$orgiden :orgchart=$position ]
+                -> ou:position
+                [ :contact=$contact :title=ceo :org=$orgiden ]
+            ''', opts=opts)
+            self.eq('ceo', nodes[0].get('title'))
+            self.eq(orgiden, nodes[0].get('org'))
+            self.eq(contact, nodes[0].get('contact'))
+
+            nodes = await core.nodes('''
+                ou:org=$orgiden
+                -> ou:position
+                [ :reports+=$subpos ]
+                -> ou:position
+            ''', opts=opts)
+            self.eq(('ou:position', subpos), nodes[0].ndef)
+
+            nodes = await core.nodes('''
+                ou:org=$orgiden
+                [ :subs+=$suborg ]
+                -> ou:org
+            ''', opts=opts)
+            self.eq(('ou:org', suborg), nodes[0].ndef)
+
             async with await core.snap() as snap:
                 guid0 = s_common.guid()
                 name = '\u21f1\u21f2 Inc.'
@@ -73,13 +158,14 @@ class OuModelTest(s_t_utils.SynTest):
                 self.eq(node.get('founded'), 1420070400000)
                 self.eq(node.get('dissolved'), 1546300800000)
 
-                nodes = (await alist(snap.getNodesBy('ou:name')))
+                nodes = await snap.nodes('ou:name')
                 self.sorteq([x.ndef[1] for x in nodes], (normname,) + altnames)
 
-                nodes = await alist(snap.getNodesBy('ou:org:names', 'otheraltarrow', cmpr='contains='))
+                nodes = await snap.nodes('ou:org:names*[=otheraltarrow]')
                 self.len(1, nodes)
 
-                nodes = await alist(snap.getNodesBy('ou:org:names', name, cmpr='contains='))
+                opts = {'var': {'name': name}}
+                nodes = await snap.nodes('ou:org:names*contains=$name', opts=opts)
                 self.len(0, nodes)  # primary ou:org:name is not in ou:org:names
 
                 person0 = s_common.guid()
@@ -105,8 +191,8 @@ class OuModelTest(s_t_utils.SynTest):
                 self.eq(node.get('perc'), 50)
                 self.eq(node.get('current'), 1)
 
-                await self.asyncraises(s_exc.BadPropValu, node.set('perc', -1))
-                await self.asyncraises(s_exc.BadPropValu, node.set('perc', 101))
+                await self.asyncraises(s_exc.BadTypeValu, node.set('perc', -1))
+                await self.asyncraises(s_exc.BadTypeValu, node.set('perc', 101))
 
                 # ou:user
                 node = await snap.addNode('ou:user', (guid0, 'arrowman'))
@@ -274,20 +360,21 @@ class OuModelTest(s_t_utils.SynTest):
             async with await core.snap() as snap:
                 for g, props in omap.items():
                     await snap.addNode('ou:org', g, props)
-                nodes = await alist(snap.getNodesBy('ou:org:sic', '01', cmpr='^='))
+
+                nodes = await snap.nodes('ou:org:sic^=01')
                 self.len(3, nodes)
 
-                nodes = await alist(snap.getNodesBy('ou:org:sic', '011', cmpr='^='))
+                nodes = await snap.nodes('ou:org:sic^=011')
                 self.len(2, nodes)
 
-                nodes = await alist(snap.getNodesBy('ou:org:naics', '22', cmpr='^='))
+                nodes = await snap.nodes('ou:org:naics^=22')
                 self.len(4, nodes)
 
-                nodes = await alist(snap.getNodesBy('ou:org:naics', '221', cmpr='^='))
+                nodes = await snap.nodes('ou:org:naics^=221')
                 self.len(4, nodes)
 
-                nodes = await alist(snap.getNodesBy('ou:org:naics', '2211', cmpr='^='))
+                nodes = await snap.nodes('ou:org:naics^=2211')
                 self.len(3, nodes)
 
-                nodes = await alist(snap.getNodesBy('ou:org:naics', '22112', cmpr='^='))
+                nodes = await snap.nodes('ou:org:naics^=22112')
                 self.len(2, nodes)
