@@ -36,9 +36,6 @@ class TelcoModelTest(s_t_utils.SynTest):
                 self.eq(node.get('org'), oguid)
                 # defvals
                 node = await snap.addNode('tel:mob:tac', 2)
-                self.eq(node.get('manu'), '??')
-                self.eq(node.get('model'), '??')
-                self.eq(node.get('internal'), '??')
 
                 # tel:mob:imid
                 node = await snap.addNode('tel:mob:imid', (490154203237518, 310150123456789))
@@ -82,10 +79,12 @@ class TelcoModelTest(s_t_utils.SynTest):
 
                 # tel:mob:telem
                 guid = s_common.guid()
+                host = s_common.guid()
                 softguid = s_common.guid()
                 props = {'time': '2001',
                          'latlong': (-1, 1),
                          'place': place,
+                         'host': host,
                          'loc': 'us',
                          'accuracy': '100mm',
                          'cell': (('001', '02'), 3, 4),
@@ -111,6 +110,7 @@ class TelcoModelTest(s_t_utils.SynTest):
                 self.eq(node.get('time'), 978307200000)
                 self.eq(node.get('latlong'), (-1.0, 1.0))
                 self.eq(node.get('place'), place)
+                self.eq(node.get('host'), host)
                 self.eq(node.get('loc'), 'us')
                 self.eq(node.get('accuracy'), 100)
                 self.eq(node.get('cell'), (('001', '02'), 3, 4))
@@ -143,9 +143,9 @@ class TelcoModelTest(s_t_utils.SynTest):
                 node = await snap.addNode('tel:mob:imei', '39015420323751')
                 self.eq(node.ndef[1], 390154203237519)
                 # Invalid checksum
-                await self.asyncraises(s_exc.BadPropValu, snap.addNode('tel:mob:imei', 490154203237519))
-                await self.asyncraises(s_exc.BadPropValu, snap.addNode('tel:mob:imei', '20'))
-                await self.asyncraises(s_exc.BadPropValu, snap.addNode('tel:mob:imei', 'hehe'))
+                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('tel:mob:imei', 490154203237519))
+                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('tel:mob:imei', '20'))
+                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('tel:mob:imei', 'hehe'))
 
     async def test_telco_imsi(self):
         async with self.getTestCore() as core:
@@ -153,8 +153,8 @@ class TelcoModelTest(s_t_utils.SynTest):
                 node = await snap.addNode('tel:mob:imsi', '310150123456789')
                 self.eq(node.ndef[1], 310150123456789)
                 self.eq(node.get('mcc'), '310')
-                await self.asyncraises(s_exc.BadPropValu, snap.addNode('tel:mob:imsi', 'hehe'))
-                await self.asyncraises(s_exc.BadPropValu, snap.addNode('tel:mob:imsi', 1111111111111111))
+                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('tel:mob:imsi', 'hehe'))
+                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('tel:mob:imsi', 1111111111111111))
 
     async def test_telco_phone(self):
         async with self.getTestCore() as core:
@@ -186,11 +186,11 @@ class TelcoModelTest(s_t_utils.SynTest):
                 self.eq(node.get('loc'), 'us')
                 node = await snap.addNode('tel:phone', '+1 (703) 555-2424')
                 # Esnap search
-                nodes = await alist(snap.getNodesBy('tel:phone', 17035552424))
+                nodes = await snap.nodes('tel:phone=17035552424')
                 self.len(1, nodes)
                 self.eq(nodes[0].ndef[1], '17035552424')
                 # Prefix search
-                nodes = await alist(snap.getNodesBy('tel:phone', '1703555*'))
+                nodes = await snap.nodes('tel:phone=1703555*')
                 self.len(2, nodes)
 
     async def test_telco_call(self):
@@ -259,4 +259,4 @@ class TelcoModelTest(s_t_utils.SynTest):
 
                 # add bad svc type
                 guid = s_common.guid()
-                await self.asyncraises(s_exc.BadPropValu, snap.addNode('tel:txtmesg', guid, {'svctype': 'foo'}))
+                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('tel:txtmesg', guid, {'svctype': 'foo'}))
