@@ -2,23 +2,37 @@ import os
 import sys
 import time
 import shutil
+import fnmatch
 import logging
 import argparse
 
 import lmdb
-import lmdb.tool
 
 import synapse.common as s_common
 
 logger = logging.getLogger(__name__)
 
-def backup(srcdir, dstdir):
+def backup(srcdir, dstdir, skipdirs=None, compact=True):
+    '''
+    Create a backup of a Synapse application.
 
+    Args:
+        srcdir (str): Path to the directory to backup.
+        dstdir (str): Path to backup target directory.
+        skipdirs (list or None): Optional list of relative directory name glob patterns to exclude from the backup.
+        compact (bool): Whether to optimize storage while copying to the destination.
+    '''
     tick = s_common.now()
 
     srcdir = s_common.reqdir(srcdir)
     dstdir = s_common.gendir(dstdir)
 
+    if skipdirs is None:
+        skipdirs = []
+
+    # Always avoid backing up temporary directories
+    skipdirs.append('**/tmp')
+
     logger.info(f'Starting backup of [{srcdir}]')
     logger.info(f'Destination dir: [{dstdir}]')
 
@@ -29,17 +43,26 @@ def backup(srcdir, dstdir):
         for name in list(dnames):
 
             srcpath = s_common.genpath(root, name)
-            dstpath = s_common.genpath(dstdir, relpath, name)
+
+            relname = os.path.join(relpath, name)
+
+            if any([fnmatch.fnmatch(relname, pattern) for pattern in skipdirs]):
+                logger.info(f'skipping dir:{srcpath}')
+                dnames.remove(name)
+                continue
+
+            dstpath = s_common.genpath(dstdir, relname)
 
             if name.endswith('.lmdb'):
                 dnames.remove(name)
-                backup_lmdb(srcpath, dstpath)
+                backup_lmdb(srcpath, dstpath, compact)
                 continue
 
             logger.info(f'making dir:{dstpath}')
             s_common.gendir(dstpath)
 
         for name in fnames:
+
             srcpath = s_common.genpath(root, name)
             # skip unix sockets etc...
             if not os.path.isfile(srcpath):
@@ -54,30 +77,26 @@ def backup(srcdir, dstdir):
     logger.info(f'Backup complete. Took [{tock-tick:.2f}] for [{srcdir}]')
     return
 
-def backup_lmdb(envpath, dstdir):
+def backup_lmdb(envpath, dstdir, compact=True):
 
     datafile = os.path.join(envpath, 'data.mdb')
     stat = os.stat(datafile)
     map_size = stat.st_size
 
-    parser = lmdb.tool.make_parser()
-    opts, args = parser.parse_args(['copy', '--compact', '-e', envpath, dstdir])
-
     env = lmdb.open(
-        opts.env,
+        envpath,
         map_size=map_size,
         subdir=True,
-        max_dbs=opts.max_dbs,
+        max_dbs=256,
         create=False,
         readonly='READ'
     )
 
-    lmdb.tool.ENV = env
     tick = time.time()
 
-    # use the builtin lmdb command
-    ret = lmdb.tool.cmd_copy(opts, args[1:])
-    logger.warning(f'lmdb ret: {ret}')
+    s_common.gendir(dstdir)
+
+    env.copy(dstdir, compact=compact)
 
     tock = time.time()
     logger.info(f'backup took: {tock-tick:.2f} seconds')
@@ -85,13 +104,16 @@ def backup_lmdb(envpath, dstdir):
 
 def main(argv):
     args = parse_args(argv)
-    backup(args.srcdir, args.dstdir)
+    backup(args.srcdir, args.dstdir, args.skipdirs)
     return 0
 
 def parse_args(argv):
-    parser = argparse.ArgumentParser()
-    parser.add_argument('srcdir', help='Path to the synapse directory to backup.')
+    desc = 'Create an optimized backup of a Synapse directory.'
+    parser = argparse.ArgumentParser('synapse.tools.backup', description=desc)
+    parser.add_argument('srcdir', help='Path to the Synapse directory to backup.')
     parser.add_argument('dstdir', help='Path to the backup target directory.')
+    parser.add_argument('--skipdirs', nargs='+',
+                        help='Glob patterns of relative directory names to exclude from the backup.')
     args = parser.parse_args(argv)
     return args
 
